"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const roles_constants_1 = require("@nestjs-bff/global-contracts/lib/shared/authorization/roles.constants");
const common_1 = require("@nestjs/common");
const access_permissions_repo_1 = require("../../domain/access-permissions/repo/access-permissions.repo");
const authentication_repo_1 = require("../../domain/authentication/repo/authentication.repo");
const encryption_util_1 = require("../../domain/authentication/utils/encryption.util");
const organization_repo_1 = require("../../domain/organization/repo/organization.repo");
const user_repo_1 = require("../../domain/user/repo/user.repo");
const crud_operations_enum_1 = require("../../shared/authchecks/crud-operations.enum");
const org_roles_authcheck_1 = require("../../shared/authchecks/org-roles.authcheck");
const app_exception_1 = require("../../shared/exceptions/app.exception");
const logger_shared_service_1 = require("../../shared/logging/logger.shared.service");
let OrganizationOrchestrationService = class OrganizationOrchestrationService {
    constructor(logger, authenticationRepo, accessPermissionsRepo, userRepo, organizationRepo) {
        this.logger = logger;
        this.authenticationRepo = authenticationRepo;
        this.accessPermissionsRepo = accessPermissionsRepo;
        this.userRepo = userRepo;
        this.organizationRepo = organizationRepo;
        // private orgUserAuthCheck = new OrgRolesAuthCheck([OrganizationRoles.member]);
        this.orgAdminAuthCheck = new org_roles_authcheck_1.OrgRolesAuthCheck([roles_constants_1.OrganizationRoles.admin]);
    }
    createMember(cmd, accessPermissions) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.trace('OrganizationOrchestrationService.createMember', { cmd, accessPermissions });
            // Authorization
            this.orgAdminAuthCheck.ensureAuthorized({
                accessPermissions,
                origin: 'OrganizationOrchestrationService',
                targetResource: { orgId: cmd.orgId },
                operation: crud_operations_enum_1.CrudOperations.create,
            });
            // setup commands
            const newAuthenticationEntity = {
                userId: undefined,
                local: {
                    email: cmd.username,
                    hashedPassword: encryption_util_1.generateHashedPassword(cmd.password),
                },
                google: undefined,
                facebook: undefined,
                twitter: undefined,
            };
            // validate
            this.authenticationRepo.entityValidator.validate(newAuthenticationEntity);
            // validate organization exists
            if (!(yield this.organizationRepo.tryFindOne({ id: cmd.orgId }, { accessPermissions }))) {
                throw new app_exception_1.AppError(`Create Member: Could not find organization for Id ${cmd.orgId}`);
            }
            // create new user
            const user = yield this.userRepo.create({
                username: cmd.username,
                displayName: cmd.displayName,
            }, { accessPermissions });
            // create authentication
            newAuthenticationEntity.userId = user.id;
            this.authenticationRepo.create(newAuthenticationEntity, { accessPermissions });
            // create authorization
            const accessPermissionsEntity = this.accessPermissionsRepo.create({
                userId: user.id,
                roles: [roles_constants_1.Roles.user],
                organizations: [
                    {
                        primary: true,
                        orgId: cmd.orgId,
                        organizationRoles: [roles_constants_1.OrganizationRoles.member],
                    },
                ],
            }, { accessPermissions });
            return accessPermissionsEntity;
        });
    }
};
OrganizationOrchestrationService = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [logger_shared_service_1.LoggerSharedService,
        authentication_repo_1.AuthenticationRepo,
        access_permissions_repo_1.AccessPermissionsRepo,
        user_repo_1.UserRepo,
        organization_repo_1.OrganizationRepo])
], OrganizationOrchestrationService);
exports.OrganizationOrchestrationService = OrganizationOrchestrationService;
//# sourceMappingURL=organization-orchestration.service.js.map