"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const roles_constants_1 = require("@nestjs-bff/global-contracts/lib/shared/authorization/roles.constants");
const common_1 = require("@nestjs/common");
const access_permissions_repo_1 = require("../../domain/access-permissions/repo/access-permissions.repo");
const authentication_repo_1 = require("../../domain/authentication/repo/authentication.repo");
const facebook_authentication_service_1 = require("../../domain/authentication/social/facebook-authentication.service");
const facebook_profile__service_1 = require("../../domain/authentication/social/facebook-profile..service");
const encryption_util_1 = require("../../domain/authentication/utils/encryption.util");
const organization_repo_1 = require("../../domain/organization/repo/organization.repo");
const user_repo_1 = require("../../domain/user/repo/user.repo");
const app_exception_1 = require("../../shared/exceptions/app.exception");
const validation_exception_1 = require("../../shared/exceptions/validation.exception");
let UserAuthService = class UserAuthService {
    constructor(
    // private readonly logger: LoggerSharedService,
    fbAuthenticationService, fbProfileService, authenticationRepo, accessPermissionsRepo, userRepo, organizationRepo) {
        this.fbAuthenticationService = fbAuthenticationService;
        this.fbProfileService = fbProfileService;
        this.authenticationRepo = authenticationRepo;
        this.accessPermissionsRepo = accessPermissionsRepo;
        this.userRepo = userRepo;
        this.organizationRepo = organizationRepo;
    }
    /**
     *
     * @param cmd
     */
    signInWithLocal(cmd) {
        return __awaiter(this, void 0, void 0, function* () {
            const authenticationEntity = yield this.authenticationRepo.findOne({ 'local.email': cmd.username }, { skipAuthorization: true });
            if (!authenticationEntity)
                throw new validation_exception_1.ValidationError(['Your login credentials were not correct']);
            if (!authenticationEntity.local)
                throw new validation_exception_1.ValidationError(['Your login credentials were not correct or you do not have an account. Perhaps you registered with social login?']);
            if (!authenticationEntity.userId)
                throw new app_exception_1.AppError('UserId can not be null');
            if (!encryption_util_1.validPassword(cmd.password, authenticationEntity.local.hashedPassword))
                throw new validation_exception_1.ValidationError(['Your login credentials were not correct']);
            const accessPermissionsEntity = yield this.accessPermissionsRepo.findOne({ userId: authenticationEntity.userId }, { skipAuthorization: true });
            if (!accessPermissionsEntity)
                throw new app_exception_1.AppError('Could not find access permission information for signIn');
            return accessPermissionsEntity;
        });
    }
    /**
     *
     * @param cmd
     */
    signUpWithLocal(cmd) {
        return __awaiter(this, void 0, void 0, function* () {
            //
            // setup commands
            //
            const newAuthenticationEntity = {
                userId: '',
                local: {
                    email: cmd.username,
                    hashedPassword: encryption_util_1.generateHashedPassword(cmd.password),
                },
                google: undefined,
                facebook: undefined,
                twitter: undefined,
            };
            //
            // validate
            //
            this.authenticationRepo.entityValidator.validate(newAuthenticationEntity);
            //
            // execute
            //
            // create new user
            const user = yield this.userRepo.create({
                username: cmd.username,
                displayName: cmd.displayName,
            }, { skipAuthorization: true });
            // create authentication
            newAuthenticationEntity.userId = user.id;
            this.authenticationRepo.create(newAuthenticationEntity, {
                skipAuthorization: true,
            });
            // create organization
            const organization = yield this.organizationRepo.create({
                name: user.username,
                slug: user.username,
            }, { skipAuthorization: true });
            // create authorization
            const accessPermissionsEntity = this.accessPermissionsRepo.create({
                userId: user.id,
                roles: [roles_constants_1.Roles.user],
                organizations: [
                    {
                        primary: true,
                        orgId: organization.id,
                        organizationRoles: [roles_constants_1.OrganizationRoles.member, roles_constants_1.OrganizationRoles.admin],
                    },
                ],
            }, { skipAuthorization: true });
            return accessPermissionsEntity;
        });
    }
    /**
     *
     * @param cmd
     */
    promoteToGroupAdmin(cmd, accessPermissions) {
        return __awaiter(this, void 0, void 0, function* () {
            const accessPermissionsEntity = yield this.accessPermissionsRepo.findById(cmd.userId, { accessPermissions });
            // Validate
            if (!accessPermissionsEntity)
                throw new app_exception_1.AppError(`Could not find accessPermissionsEntity for userId ${cmd.userId}`);
            if (accessPermissionsEntity.roles.includes(roles_constants_1.Roles.groupAdmin))
                throw new app_exception_1.AppError(`User already GroupAdmin (userId ${cmd.userId})`);
            // Update
            accessPermissionsEntity.roles.push(roles_constants_1.Roles.groupAdmin);
            // Persist
            return this.accessPermissionsRepo.update(accessPermissionsEntity, { accessPermissions }).then(() => accessPermissionsEntity);
        });
    }
    /**
     *
     * @param fbAuthorizationCode
     * @param spaRootUrl
     */
    signUpWithFacebook(fbAuthorizationCode, spaRootUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            // get fb auth token using fb access token
            const fbAuthorizationToken = yield this.fbAuthenticationService.getOauthAccessToken(fbAuthorizationCode, spaRootUrl);
            // get fb profile
            const fbProfile = yield this.fbProfileService.getProfile(fbAuthorizationToken);
            // find Authentication Entity
            const authenticationEntity = yield this.authenticationRepo.findOne({ facebook: { id: fbProfile.id } }, { skipAuthorization: true });
            if (authenticationEntity)
                throw new app_exception_1.AppError('Could not find authorization information for signIn');
            // create new user
            const user = yield this.userRepo.create({
                username: fbProfile.email,
                displayName: fbProfile.name,
            }, { skipAuthorization: true });
            // create authentication
            this.authenticationRepo.create({
                userId: user.id,
                local: undefined,
                facebook: {
                    id: fbProfile.id,
                    email: fbProfile.email,
                    name: fbProfile.name,
                },
                google: undefined,
                twitter: undefined,
            }, { skipAuthorization: true });
            // create authorization
            const accessPermissionsEntity = this.accessPermissionsRepo.create({
                userId: user.id,
                roles: [roles_constants_1.Roles.user],
                organizations: [],
            }, { skipAuthorization: true });
            return accessPermissionsEntity;
        });
    }
    /**
     *
     * @param fbAuthorizationCode
     * @param spaRootUrl
     */
    signInWithFacebook(fbAuthorizationCode, spaRootUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            // get fb auth token using fb access token
            const fbAuthorizationToken = yield this.fbAuthenticationService.getOauthAccessToken(fbAuthorizationCode, spaRootUrl);
            // get fb profile
            const fbProfile = yield this.fbProfileService.getProfile(fbAuthorizationToken);
            // find Authentication Entity
            const authenticationEntity = yield this.authenticationRepo.findOne({ facebook: { id: fbProfile.id } }, { skipAuthorization: true });
            if (!authenticationEntity)
                throw new app_exception_1.AppError('Could not find authentication information for signIn');
            const accessPermissionsEntity = yield this.accessPermissionsRepo.findOne({ userId: authenticationEntity.userId }, { skipAuthorization: true });
            if (!accessPermissionsEntity)
                throw new app_exception_1.AppError('Could not find authorization information for signIn');
            return accessPermissionsEntity;
        });
    }
};
UserAuthService = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [facebook_authentication_service_1.FacebookAuthenticationService,
        facebook_profile__service_1.FacebookProfileService,
        authentication_repo_1.AuthenticationRepo,
        access_permissions_repo_1.AccessPermissionsRepo,
        user_repo_1.UserRepo,
        organization_repo_1.OrganizationRepo])
], UserAuthService);
exports.UserAuthService = UserAuthService;
//# sourceMappingURL=user-auth.service.js.map