"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const log_levels_const_1 = require("../shared/logging/log-levels.const");
const nestjs_config_test_1 = require("./nestjs.config.test");
const _NestjsBffConfig = {
    orgName: 'my-org',
    appName: 'my-app',
    rootPath: process.cwd(),
    caching: {
        entities: {
            organization: 60 * 20,
            user: 60 * 20,
            authorization: 60 * 20,
        },
    },
    migrations: {
        autoRun: false,
    },
    db: {
        mongo: {
            debugLogging: true,
            options: {
                useNewUrlParser: true,
                useCreateIndex: true,
                useFindAndModify: false,
                autoIndex: false,
                reconnectTries: Number.MAX_VALUE,
                reconnectInterval: 500,
                poolSize: 10,
                bufferMaxEntries: 0,
                connectTimeoutMS: 10000,
                socketTimeoutMS: 45000,
                family: 4,
            },
        },
    },
    http: {
        publicRouteRegex: /public\//,
    },
    jwt: {
        jwtPrivateKey: '',
        jwtPrivateKeyPemPassphrase: 'D161tal',
        jwtPublicKey: '',
        issuer: 'entity',
        expiresIn: '18h',
        signingAlgorithm: 'RS256',
    },
    logging: {
        logDir: 'logs',
        console: {
            levels: [log_levels_const_1.LogLevels.error, log_levels_const_1.LogLevels.warning, log_levels_const_1.LogLevels.info, log_levels_const_1.LogLevels.debug, log_levels_const_1.LogLevels.trace],
        },
        winston: {
            level: 'info',
            enableMiddleware: true,
            transports: {
                console: {
                    colorize: true,
                    timestamp: true,
                    json: false,
                    showLevel: true,
                },
                file: {
                    json: true,
                    maxsize: 10485760,
                    maxfile: 5,
                },
            },
            target: {
                trace: 'trace.log',
                debug: 'debug.log',
                error: 'error.log',
                warn: 'info.log',
                info: 'info.log',
            },
        },
    },
    social: {
        facebook: {
            authorizeHost: 'https://facebook.com',
            tokenHost: 'https://graph.facebook.com/v3.1',
            apiHost: 'https://graph.facebook.com/v3.1',
            authorizePath: '/dialog/oauth',
            authorizeScope: ['email'],
            tokenPath: '/oauth/access_token',
            apiProfilePath: '/me',
            callbackRelativeURL_signIn: '/auth/facebook/sign-in',
            callbackRelativeURL_signUp: '/auth/facebook/sign-up',
        },
    },
};
exports.NestjsBffConfig = _.merge(_NestjsBffConfig, nestjs_config_test_1.NestjsBffConfigEnv);
//# sourceMappingURL=nestjs-bff.config.js.map