"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const roles_constants_1 = require("@nestjs-bff/global-contracts/lib/shared/authorization/roles.constants");
const authcheck_contract_1 = require("../../../shared/authchecks/authcheck.contract");
const org_roles_authcheck_1 = require("../../../shared/authchecks/org-roles.authcheck");
const scoped_data_1 = require("../../../shared/authchecks/scoped-data");
const app_exception_1 = require("../../../shared/exceptions/app.exception");
//
// checks permissions against orgId and userId if these attributes exist on an entity
//
class AccessPermissionsEntityAuthCheck extends authcheck_contract_1.AuthCheckContract {
    constructor() {
        super(...arguments);
        this.orgAdminAuthCheck = new org_roles_authcheck_1.OrgRolesAuthCheck([roles_constants_1.OrganizationRoles.admin]);
    }
    isAuthorized(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!params.targetResource || !params.targetResource)
                throw new app_exception_1.AppError('target.resource can not be null');
            const scopedParams = {
                accessPermissions: params.accessPermissions,
                origin: params.origin,
                operation: params.operation,
                targetResource: new scoped_data_1.ScopedData(),
                data: params.data,
            };
            if (params.targetResource.organizations) {
                for (const orgAccess of params.targetResource.organizations) {
                    scopedParams.targetResource.orgId = orgAccess.orgId;
                    yield this.orgAdminAuthCheck.ensureAuthorized(scopedParams);
                }
            }
            // if authorization not already denied, then allow access (return true)
            return true;
        });
    }
}
exports.AccessPermissionsEntityAuthCheck = AccessPermissionsEntityAuthCheck;
//# sourceMappingURL=access-permissions-entity.authcheck.js.map