"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const simple_oauth2_1 = require("simple-oauth2");
const app_shared_constants_1 = require("../../../shared/app/app.shared.constants");
const app_exception_1 = require("../../../shared/exceptions/app.exception");
const logger_shared_service_1 = require("../../../shared/logging/logger.shared.service");
let FacebookAuthenticationService = class FacebookAuthenticationService {
    constructor(nestjsBffConfig, bffLoggerService) {
        this.nestjsBffConfig = nestjsBffConfig;
        this.bffLoggerService = bffLoggerService;
        const oauthClientConfig = {
            client: {
                id: this.nestjsBffConfig.social.facebook.clientID,
                secret: this.nestjsBffConfig.social.facebook.clientSecret,
            },
            auth: {
                authorizeHost: this.nestjsBffConfig.social.facebook.authorizeHost,
                authorizePath: this.nestjsBffConfig.social.facebook.authorizePath,
                tokenHost: this.nestjsBffConfig.social.facebook.tokenHost,
                tokenPath: this.nestjsBffConfig.social.facebook.tokenPath,
            },
        };
        // this.bffLoggerService.debug('FacebookAuthenticationService-init-oauthClientConfig', oauthClientConfig);
        this.oauthClient = simple_oauth2_1.create(oauthClientConfig);
    }
    /**
     *
     * @param spaRootUrl
     */
    getOauth2RedirectUrl(redirectUri) {
        return __awaiter(this, void 0, void 0, function* () {
            const options = {
                redirect_uri: `${redirectUri}`,
                scope: this.nestjsBffConfig.social.facebook.authorizeScope.join(','),
                state: '',
            };
            const redirectUrl = this.oauthClient.authorizationCode.authorizeURL(options);
            return {
                redirectUrl,
            };
        });
    }
    /**
     *
     * @param code
     */
    getOauthAccessToken(fbAuthorizationCode, spaRootUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const tokenConfig = {
                code: `${fbAuthorizationCode}`,
                redirect_uri: `${spaRootUrl}${this.nestjsBffConfig.social.facebook.callbackRelativeURL_signIn}`,
                scope: this.nestjsBffConfig.social.facebook.authorizeScope,
            };
            try {
                // Get access token
                const result = yield this.oauthClient.authorizationCode.getToken(tokenConfig);
                return { token: result.access_token };
            }
            catch (error) {
                this.bffLoggerService.error('Error retrieving Facebook oAuth access token', error);
                throw new app_exception_1.AppError('There was a problem authenticating you with Facebook');
            }
        });
    }
};
FacebookAuthenticationService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(app_shared_constants_1.AppSharedProviderTokens.Config.App)),
    __metadata("design:paramtypes", [Object, logger_shared_service_1.LoggerSharedService])
], FacebookAuthenticationService);
exports.FacebookAuthenticationService = FacebookAuthenticationService;
//# sourceMappingURL=facebook-authentication.service.js.map