"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const testing_utils_1 = require("@nestjs-bff/global-utils-dev/lib/testing.utils");
const cacheManager = require("cache-manager");
const _ = require("lodash");
const mongoose = require("mongoose");
const nestjs_bff_config_1 = require("../../../config/nestjs-bff.config");
const app_exception_1 = require("../../../shared/exceptions/app.exception");
const authorization_exception_1 = require("../../../shared/exceptions/authorization.exception");
const validation_exception_1 = require("../../../shared/exceptions/validation.exception");
const console_logger_shared_service_1 = require("../../../shared/logging/console-logger.shared.service");
const logging_shared_module_1 = require("../../../shared/logging/logging.shared.module");
const test_literals_constants_1 = require("../../../shared/testing/test-literals.constants");
const foo_schema_1 = require("../../_foo/model/foo.schema");
const foo_repo_1 = require("../../_foo/repo/foo.repo");
//
// Global Scoped Variables Setup
//
// @ts-ignore
const logger = logging_shared_module_1.getLogger();
describe('GIVEN a Repo', () => {
    let fooRepo;
    let memCache;
    let loggerService;
    let fooModel;
    beforeAll(() => __awaiter(this, void 0, void 0, function* () {
        loggerService = new console_logger_shared_service_1.LoggerConsoleSharedService(nestjs_bff_config_1.NestjsBffConfig);
        fooModel = mongoose.model('Foo', foo_schema_1.FooSchema);
    }));
    //
    // -------------------------------------------
    //
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        memCache = cacheManager.caching({
            store: 'memory',
            max: 100,
            ttl: 10 /*seconds*/,
        });
        fooRepo = new foo_repo_1.FooRepo(loggerService, nestjs_bff_config_1.NestjsBffConfig, memCache, fooModel);
    }));
    //
    // -------------------------------------------
    //
    // FindById Tests
    //
    // -------------------------------------------
    //
    describe('WHEN findById is called with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN a Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(id => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                result = yield fooRepo.findById(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toEqual(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH valid authorization
        FOR an entity that does not exist 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(id => {
                return null;
            });
            try {
                result = yield fooRepo.findById(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(app_exception_1.AppError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(id => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                result = yield fooRepo.findById(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id);
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization
        WITH options.skipAuthorization = true 
        THEN an Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(id => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                result = yield fooRepo.findById(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { skipAuthorization: true });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toEqual(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
        }));
    });
    //
    // -------------------------------------------
    // -------------------------------------------
    //
    describe('WHEN findOne is called twice with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN _dbFindOne should only be triggered once (the other is from cache) `, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            // @ts-ignore
            const spyDbFindOne = jest.spyOn(fooRepo, '_dbFindById').mockImplementation(conditions => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                yield fooRepo.findById(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
                yield fooRepo.findById(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(spyDbFindOne).toBeCalledTimes(1);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH valid authorization
        AND options.skipCache = true 
        THEN _dbFindOne should be triggered twice (cache not used) `, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            // @ts-ignore
            const spyDbFindOne = jest.spyOn(fooRepo, '_dbFindById').mockImplementation(conditions => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                yield fooRepo.findById(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember, skipCache: true });
                yield fooRepo.findById(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember, skipCache: true });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(spyDbFindOne).toBeCalledTimes(2);
        }));
    });
    //
    // -------------------------------------------
    //
    // -------------------------------------------
    //
    // FindOne Tests
    //
    // -------------------------------------------
    //
    describe('WHEN findOne is called with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN a Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindOne').mockImplementation(conditions => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                result = yield fooRepo.findOne({ alwaysDefinedSlug: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.alwaysDefinedSlug }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toEqual(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH valid authorization
        FOR an entity that does not exist 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindOne').mockImplementation(conditions => {
                return null;
            });
            try {
                result = yield fooRepo.findOne({ alwaysDefinedSlug: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.alwaysDefinedSlug }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(app_exception_1.AppError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindOne').mockImplementation(conditions => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                result = yield fooRepo.findOne({ alwaysDefinedSlug: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.alwaysDefinedSlug });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization
        WITH options.skipAuthorization = true 
        THEN an Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindOne').mockImplementation(conditions => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                result = yield fooRepo.findOne({ alwaysDefinedSlug: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.alwaysDefinedSlug }, { skipAuthorization: true });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toEqual(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
        }));
    });
    //
    // -------------------------------------------
    // -------------------------------------------
    //
    describe('WHEN findOne is called twice with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN _dbFindOne should only be triggered once (the other is from cache) `, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            // @ts-ignore
            const spyDbFindOne = jest.spyOn(fooRepo, '_dbFindOne').mockImplementation(conditions => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                yield fooRepo.findOne({ alwaysDefinedSlug: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.alwaysDefinedSlug }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
                yield fooRepo.findOne({ alwaysDefinedSlug: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.alwaysDefinedSlug }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(spyDbFindOne).toBeCalledTimes(1);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH valid authorization
        AND options.skipCache = true 
        THEN _dbFindOne should be triggered twice (cache not used) `, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            // @ts-ignore
            const spyDbFindOne = jest.spyOn(fooRepo, '_dbFindOne').mockImplementation(conditions => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            try {
                yield fooRepo.findOne({ alwaysDefinedSlug: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.alwaysDefinedSlug }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember, skipCache: true });
                yield fooRepo.findOne({ alwaysDefinedSlug: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.alwaysDefinedSlug }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember, skipCache: true });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(spyDbFindOne).toBeCalledTimes(2);
        }));
    });
    //
    // -------------------------------------------
    //
    //
    // -------------------------------------------
    //
    // Find Tests
    //
    // -------------------------------------------
    //
    describe('WHEN find is called with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN a Foo array should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFind').mockImplementation(conditions => {
                return [test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa];
            });
            try {
                result = yield fooRepo.find({ name: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.name }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toEqual([test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa]);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH valid authorization
        FOR no matching entities 
        THEN an empty array should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFind').mockImplementation(conditions => {
                return [];
            });
            try {
                result = yield fooRepo.find({ id: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toEqual([]);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFind').mockImplementation(conditions => {
                return [test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa];
            });
            try {
                result = yield fooRepo.find({ name: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.name });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization
        WITH options.skipAuthorization = true 
        THEN an Foo array should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFind').mockImplementation(conditions => {
                return [test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa];
            });
            try {
                result = yield fooRepo.find({ name: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.name }, { skipAuthorization: true });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toEqual([test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa]);
        }));
    });
    //
    // -------------------------------------------
    // -------------------------------------------
    //
    describe('WHEN find is called twice with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN _dbFind should only be triggered once (the other is from cache) `, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            // @ts-ignore
            const spyDbFind = jest.spyOn(fooRepo, '_dbFind').mockImplementation(conditions => {
                return [test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa];
            });
            try {
                yield fooRepo.find({ name: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.name }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
                yield fooRepo.find({ name: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.name }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(spyDbFind).toBeCalledTimes(1);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH valid authorization
        AND options.skipCache = true 
        THEN _dbFind should be triggered twice (cache not used) `, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            // @ts-ignore
            const spyDbFind = jest.spyOn(fooRepo, '_dbFind').mockImplementation(conditions => {
                return [test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa];
            });
            try {
                yield fooRepo.find({ name: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.name }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember, skipCache: true });
                yield fooRepo.find({ name: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.name }, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember, skipCache: true });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(spyDbFind).toBeCalledTimes(2);
        }));
    });
    //
    // -------------------------------------------
    //
    // Create Tests
    //
    // -------------------------------------------
    //
    describe('WHEN create is called with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN a Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbSave').mockImplementation(entity => {
                return entity;
            });
            const newFoo = _.clone(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
            newFoo.id = '';
            try {
                result = yield fooRepo.create(newFoo, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            // delete id, to prep object for comparison
            delete newFoo.id;
            expect(error).toBeUndefined();
            expect(testing_utils_1.TestingUtils.objectIdsToStrings(result)).toMatchObject(newFoo);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbSave').mockImplementation(entity => {
                return entity;
            });
            const newFoo = _.clone(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
            newFoo.id = '';
            try {
                result = yield fooRepo.create(newFoo);
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH invalid authorization 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbSave').mockImplementation(entity => {
                return entity;
            });
            const newFoo = _.clone(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
            newFoo.id = '';
            try {
                result = yield fooRepo.create(newFoo, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ub1user_ObAdmin });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
    });
    //
    // -------------------------------------------
    //
    // Patch Tests
    //
    // -------------------------------------------
    //
    describe('WHEN patch is called with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN a Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            const fooToBeUpdated = _.clone(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(entityId => {
                return fooToBeUpdated;
            });
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbSave').mockImplementation(entity => {
                return entity;
            });
            const patchFoo = {
                id: fooToBeUpdated.id,
                name: fooToBeUpdated.name + ' updated',
            };
            try {
                result = yield fooRepo.patch(patchFoo, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            const expectedFoo = fooToBeUpdated;
            expectedFoo.name = expectedFoo.name + ' updated';
            expect(error).toBeUndefined();
            expect(testing_utils_1.TestingUtils.objectIdsToStrings(result)).toMatchObject(expectedFoo);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            const fooToBeUpdated = _.clone(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(entityId => {
                return fooToBeUpdated;
            });
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbSave').mockImplementation(entity => {
                return entity;
            });
            const patchFoo = {
                id: fooToBeUpdated.id,
                name: fooToBeUpdated.name + ' updated',
            };
            try {
                result = yield fooRepo.patch(patchFoo);
            }
            catch (e) {
                error = e;
            }
            const expectedFoo = fooToBeUpdated;
            expectedFoo.name = expectedFoo.name + ' updated';
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH invalid authorization 
        THEN an error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            const fooToBeUpdated = _.clone(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(entityId => {
                return fooToBeUpdated;
            });
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbSave').mockImplementation(entity => {
                return entity;
            });
            const patchFoo = {
                id: fooToBeUpdated.id,
                name: fooToBeUpdated.name + ' updated',
            };
            try {
                result = yield fooRepo.patch(patchFoo, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ub1user_ObAdmin });
            }
            catch (e) {
                error = e;
                // logger.debug('patch error', JSON.stringify(error, null, 2));
            }
            const expectedFoo = fooToBeUpdated;
            expectedFoo.name = expectedFoo.name + ' updated';
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
    });
    //
    // -------------------------------------------
    //
    // Update Tests
    //
    // -------------------------------------------
    //
    describe('WHEN update is called with an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN a Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindOneAndReplace').mockImplementation(entity => {
                return entity;
            });
            try {
                result = yield fooRepo.update(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(testing_utils_1.TestingUtils.objectIdsToStrings(result)).toMatchObject(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no orgId 
        THEN a validation error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindOneAndReplace').mockImplementation(entity => {
                return entity;
            });
            const fooWithMissingOrgId = _.clone(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
            delete fooWithMissingOrgId.orgId;
            try {
                result = yield fooRepo.update(fooWithMissingOrgId, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(validation_exception_1.ValidationError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization 
        THEN an authorization error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindOneAndReplace').mockImplementation(entity => {
                return entity;
            });
            try {
                result = yield fooRepo.update(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH invalid authorization 
        THEN an authorization error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindOneAndReplace').mockImplementation(entity => {
                return entity;
            });
            try {
                result = yield fooRepo.update(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ub1user_ObAdmin });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
    });
    //
    // -------------------------------------------
    //
    // Delete Tests
    //
    // -------------------------------------------
    //
    describe('WHEN delete is called with an id for an org-scoped and user-scoped Repo', () => {
        it(`WITH valid authorization 
        THEN a Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(entityId => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbRemove').mockImplementation(entity => {
                return entity;
            });
            try {
                result = yield fooRepo.delete(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(testing_utils_1.TestingUtils.objectIdsToStrings(result)).toMatchObject(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH valid groupAdmin authorization 
    THEN a Foo should be returned`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(entityId => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbRemove').mockImplementation(entity => {
                return entity;
            });
            try {
                result = yield fooRepo.delete(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Uc1GroupAdmin_OcMember_OaFacilitator });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(testing_utils_1.TestingUtils.objectIdsToStrings(result)).toMatchObject(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH no authorization 
        THEN an authorization error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(entityId => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbRemove').mockImplementation(entity => {
                return entity;
            });
            try {
                result = yield fooRepo.delete(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id);
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
        //
        // -------------------------------------------
        //
        it(`WITH invalid authorization 
        THEN an authorization error should be thrown`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbFindById').mockImplementation(entityId => {
                return test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa;
            });
            // @ts-ignore
            jest.spyOn(fooRepo, '_dbRemove').mockImplementation(entity => {
                return entity;
            });
            try {
                result = yield fooRepo.delete(test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa.id, { accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ub1user_ObAdmin });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeInstanceOf(authorization_exception_1.AuthorizationError);
            expect(result).toBeUndefined();
        }));
    });
});
//
// Unused Snippets
//
// Instantiate Object Via NestJS
/*
  const module = await Test.createTestingModule({
    imports: [DomainCoreModule, DomainFooModule],
  }).compile();

  fooRepo = await module.get<FooRepo>(FooRepo);
*/
//# sourceMappingURL=base.repo.spec.js.map