"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const class_validator_1 = require("class-validator");
const _ = require("lodash");
const validation_exception_1 = require("../../../shared/exceptions/validation.exception");
const logger_shared_service_1 = require("../../../shared/logging/logger.shared.service");
let ClassValidator = class ClassValidator {
    constructor(loggerService, entityType, coalesceType = true) {
        this.loggerService = loggerService;
        this.entityType = entityType;
        this.coalesceType = coalesceType;
    }
    validate(obj, options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loggerService.trace(`ClassValidator.validate`, obj);
            const validationError = yield this.tryValidate(obj, options);
            if (validationError) {
                throw validationError;
            }
        });
    }
    tryValidate(obj, options) {
        return __awaiter(this, void 0, void 0, function* () {
            options = options || {};
            let validationError = null;
            // prepare class
            obj = this.prepareClassForValidation(obj);
            // validate
            const classValidatorValidationErrors = yield class_validator_1.validate(obj, {
                skipMissingProperties: options.skipMissingProperties !== undefined ? options.skipMissingProperties : true,
                groups: options.validationGroups || [],
            });
            // if validation errors, create Exception object
            if (classValidatorValidationErrors && classValidatorValidationErrors.length > 0) {
                const validationErrorMessages = [];
                classValidatorValidationErrors.forEach(valErr => {
                    validationErrorMessages.push(`Property '${valErr.property}' can not contain value '${valErr.value}'`);
                });
                validationError = new validation_exception_1.ValidationError(validationErrorMessages, { target: obj, options, data: classValidatorValidationErrors });
            }
            // return exception object
            return validationError;
        });
    }
    prepareClassForValidation(obj) {
        if (!this.coalesceType)
            return obj;
        // ensure the object has the relevant attributes - as typescript has structural typing, not nominal typing
        const entityWithAttributes = new this.entityType();
        _.merge(entityWithAttributes, obj);
        return entityWithAttributes;
    }
};
ClassValidator = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [logger_shared_service_1.LoggerSharedService, Object, Boolean])
], ClassValidator);
exports.ClassValidator = ClassValidator;
class ValidationOptions {
}
exports.ValidationOptions = ValidationOptions;
//# sourceMappingURL=class-validator.js.map