"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const validation_exception_1 = require("../../../shared/exceptions/validation.exception");
const logging_shared_module_1 = require("../../../shared/logging/logging.shared.module");
const testing_utils_1 = require("@nestjs-bff/global-utils-dev/lib/testing.utils");
const foo_entity_1 = require("../../_foo/model/foo.entity");
const class_validator_1 = require("./class-validator");
// @ts-ignore
const logger = logging_shared_module_1.getLogger();
describe('GIVEN ClassValidator', () => {
    const entityValidator = new class_validator_1.ClassValidator(logger, foo_entity_1.FooEntity);
    describe('validateEntity', () => {
        describe('UserAndOrgScopedEntityConditions', () => {
            it('WHEN all required parameters are provided, THEN should pass', () => __awaiter(this, void 0, void 0, function* () {
                const foo = {
                    id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    orgId: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    userId: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    alwaysDefinedSlug: 'fooman',
                };
                let error;
                try {
                    yield entityValidator.validate(foo);
                }
                catch (e) {
                    error = e;
                    // logger.debug('error', { error, innerErrors: error.metaData.errors });
                }
                expect(error).toBeUndefined();
            }));
            it('WHEN alwaysDefinedSlug is missing THEN should throw error', () => __awaiter(this, void 0, void 0, function* () {
                const fooEntity = {
                    id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    orgId: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    userId: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    name: 'Mr Fooman',
                };
                let error;
                try {
                    yield entityValidator.validate(fooEntity);
                }
                catch (e) {
                    error = e;
                }
                expect(error).toBeInstanceOf(validation_exception_1.ValidationError);
            }));
            it('WHEN name is less than 5 characters THEN should throw error', () => __awaiter(this, void 0, void 0, function* () {
                const fooEntity = {
                    id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    orgId: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    userId: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    name: 'Foo',
                    alwaysDefinedSlug: 'fooman',
                };
                let error;
                try {
                    yield entityValidator.validate(fooEntity);
                }
                catch (e) {
                    error = e;
                }
                expect(error).toBeInstanceOf(validation_exception_1.ValidationError);
            }));
            it('WHEN userId and orgId are undefined THEN should still pass', () => __awaiter(this, void 0, void 0, function* () {
                const fooEntity = {
                    id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    alwaysDefinedSlug: 'fooman',
                    userId: undefined,
                };
                let error;
                try {
                    yield entityValidator.validate(fooEntity);
                }
                catch (e) {
                    error = e;
                    // logger.debug('error', { error, innerErrors: error.metaData.errors });
                }
                expect(error).toBeUndefined();
            }));
            it(`WHEN userId and orgId are undefined 
          AND validationOptions.skipMissingProperties = false
          THEN should throw error`, () => __awaiter(this, void 0, void 0, function* () {
                const fooEntity = {
                    id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    alwaysDefinedSlug: 'fooman',
                    name: 'Mr Fooman',
                };
                let error;
                try {
                    yield entityValidator.validate(fooEntity, { skipMissingProperties: false });
                }
                catch (e) {
                    error = e;
                    // logger.debug('error', JSON.stringify(error, null, 2));
                }
                expect(error).toBeInstanceOf(validation_exception_1.ValidationError);
            }));
            it(`WHEN orgId is undefined 
          AND validationOptions.skipMissingProperties = false
          THEN should throw error`, () => __awaiter(this, void 0, void 0, function* () {
                const fooEntity = {
                    id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    alwaysDefinedSlug: 'fooman',
                    userId: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                    name: 'Mr Fooman',
                };
                let error;
                try {
                    yield entityValidator.validate(fooEntity, { skipMissingProperties: false });
                }
                catch (e) {
                    error = e;
                    // logger.debug('error', JSON.stringify(error, null, 2));
                }
                expect(error).toBeInstanceOf(validation_exception_1.ValidationError);
            }));
        });
    });
});
//# sourceMappingURL=class-validator.spec.js.map