"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const scoped_entity_authcheck_1 = require("../../../../shared/authchecks/scoped-entity.authcheck");
const authorization_decorator_1 = require("../decorators/authorization.decorator");
/*
  Domain Service Hosted Endpoints are RESTful, with the following best-practice structure

  GET /items - Retrieves a list of items
  GET /items/12 - Retrieves a specific item
  POST /items - Creates a new item
  PUT /items/12 - Updates item #12
  PATCH /items/12 - Partially updates item #12
  DELETE /items/12 - Deletes item #12

*/
class DomainControllerBase {
    constructor(entityRepo) {
        this.entityRepo = entityRepo;
    }
    getItems(req, body) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.entityRepo.find(body || {}, { accessPermissions: req.accessPermissions });
        });
    }
    getItem(req, id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.entityRepo.findById(id, { accessPermissions: req.accessPermissions });
        });
    }
    create(req, entity) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.entityRepo.create(entity, { accessPermissions: req.accessPermissions });
        });
    }
    update(req, entity) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.entityRepo.update(entity, { accessPermissions: req.accessPermissions });
        });
    }
    partialUpdate(req, entity) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.entityRepo.patch(entity, { accessPermissions: req.accessPermissions });
        });
    }
    delete(req, id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.entityRepo.delete(id, { accessPermissions: req.accessPermissions });
        });
    }
}
__decorate([
    common_1.Get(),
    authorization_decorator_1.Authorization([new scoped_entity_authcheck_1.ScopedEntityAuthCheck()]),
    __param(0, common_1.Req()), __param(1, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], DomainControllerBase.prototype, "getItems", null);
__decorate([
    common_1.Get(':id'),
    authorization_decorator_1.Authorization([new scoped_entity_authcheck_1.ScopedEntityAuthCheck()]),
    __param(0, common_1.Req()), __param(1, common_1.Param('id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DomainControllerBase.prototype, "getItem", null);
__decorate([
    common_1.Post(),
    authorization_decorator_1.Authorization([new scoped_entity_authcheck_1.ScopedEntityAuthCheck()]),
    __param(0, common_1.Req()), __param(1, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], DomainControllerBase.prototype, "create", null);
__decorate([
    common_1.Put(),
    authorization_decorator_1.Authorization([new scoped_entity_authcheck_1.ScopedEntityAuthCheck()]),
    __param(0, common_1.Req()), __param(1, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], DomainControllerBase.prototype, "update", null);
__decorate([
    common_1.Patch(),
    authorization_decorator_1.Authorization([new scoped_entity_authcheck_1.ScopedEntityAuthCheck()]),
    __param(0, common_1.Req()), __param(1, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], DomainControllerBase.prototype, "partialUpdate", null);
__decorate([
    common_1.Delete(),
    authorization_decorator_1.Authorization([new scoped_entity_authcheck_1.ScopedEntityAuthCheck()]),
    __param(0, common_1.Req()), __param(1, common_1.Param('id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], DomainControllerBase.prototype, "delete", null);
exports.DomainControllerBase = DomainControllerBase;
//# sourceMappingURL=domain-controller-base.js.map