"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const roles_constants_1 = require("@nestjs-bff/global-contracts/lib/shared/authorization/roles.constants");
const jsonwebtoken_1 = require("jsonwebtoken");
const nestjs_bff_config_1 = require("../../../../config/nestjs-bff.config");
const logging_shared_module_1 = require("../../../../shared/logging/logging.shared.module");
const jwt_token_service_1 = require("./jwt-token.service");
// @ts-ignore
const logger = logging_shared_module_1.getLogger();
const verifyOptions = {
    issuer: nestjs_bff_config_1.NestjsBffConfig.jwt.issuer,
    audience: nestjs_bff_config_1.NestjsBffConfig.http.bffRootUrl,
    algorithms: [nestjs_bff_config_1.NestjsBffConfig.jwt.signingAlgorithm],
};
describe('JwtTokenService', () => {
    let jwtTokenService;
    beforeAll(() => __awaiter(this, void 0, void 0, function* () {
        nestjs_bff_config_1.NestjsBffConfig.jwt.jwtPrivateKey = `-----BEGIN RSA PRIVATE KEY-----
MIIBOgIBAAJBALA6Ud1u8QCBV4X59m4Puu1vpnOie9GsNHAlRUQ161mqPZh/hg4A
RYQoSkwmwZHA4xvEcYeyjm0GPkK5nKtI8RcCAwEAAQJACp5pAmvF797bVEJcnAMs
o1P/9qXKyjaTLlLAmryZAt0KoFBS4Ziaa7M0ItlUeuzZv6xFlAU+ARlUmsOej2Pr
OQIhANmR2ev01axiyo82Xa0qJzeL2IPQgNmVP7M22ARMOLutAiEAz1sN5SwxqcVm
0TACjzwD8SbmueZ23+4vEBB4Ko33eFMCIQCaINvLby+rpnSuzanBEZqkm/ovLxcI
jNWKhPC04rZSJQIgSGguWPluui7hcWjHbAb0BXClHwNYPWfp7T0jCREb+lsCIAJ4
XZYzfe2+VUcYQHG96tJaOrq8pOKySRFw+rBcEGLa
-----END RSA PRIVATE KEY-----`;
        nestjs_bff_config_1.NestjsBffConfig.jwt.jwtPublicKey = `-----BEGIN PUBLIC KEY-----
MFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBALA6Ud1u8QCBV4X59m4Puu1vpnOie9Gs
NHAlRUQ161mqPZh/hg4ARYQoSkwmwZHA4xvEcYeyjm0GPkK5nKtI8RcCAwEAAQ==
-----END PUBLIC KEY-----`;
        jwtTokenService = new jwt_token_service_1.JwtTokenService(nestjs_bff_config_1.NestjsBffConfig);
        // logger.debug('NestjsBffConfig.jwt.jwtPrivateKey', NestjsBffConfig.jwt.jwtPrivateKey);
    }));
    describe('createToken', () => {
        it('should return a jwt Token', () => __awaiter(this, void 0, void 0, function* () {
            const accessPermissionsEntity = {
                id: '507f1f77bcf86cd799439011',
                roles: [roles_constants_1.Roles.user],
            };
            const authToken = yield jwtTokenService.createToken(accessPermissionsEntity);
            expect(authToken).toBeDefined();
            expect(authToken).toHaveProperty('token');
            // decode token
            const decodedJwtToken = jsonwebtoken_1.verify(authToken.token, nestjs_bff_config_1.NestjsBffConfig.jwt.jwtPublicKey, verifyOptions);
            expect(decodedJwtToken).toBeDefined();
            expect(decodedJwtToken).toHaveProperty('iat');
            expect(decodedJwtToken.iat).not.toBeNull();
            expect(decodedJwtToken).toHaveProperty('exp');
            expect(decodedJwtToken.exp).not.toBeNull();
            expect(decodedJwtToken).toHaveProperty('aud');
            expect(decodedJwtToken.aud).not.toBeNull();
            expect(decodedJwtToken).toHaveProperty('iss');
            expect(decodedJwtToken.iss).not.toBeNull();
            expect(decodedJwtToken).toHaveProperty('sub');
            expect(decodedJwtToken.sub).not.toBeNull();
            expect(decodedJwtToken).toHaveProperty('roles');
            expect(decodedJwtToken.roles).not.toBeNull();
        }));
    });
});
//# sourceMappingURL=jwt-token.service.spec.js.map