"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
var AttachAuthenticationHttpMiddleware_1;
"use strict";
const common_1 = require("@nestjs/common");
const jsonwebtoken_1 = require("jsonwebtoken");
const access_permissions_repo_1 = require("../../../../domain/access-permissions/repo/access-permissions.repo");
const app_shared_constants_1 = require("../../../../shared/app/app.shared.constants");
const logger_shared_service_1 = require("../../../../shared/logging/logger.shared.service");
const server_exception_1 = require("../exceptions/server.exception");
const core_utils_1 = require("../utils/core.utils");
let AttachAuthenticationHttpMiddleware = AttachAuthenticationHttpMiddleware_1 = class AttachAuthenticationHttpMiddleware {
    constructor(bffLoggerService, nestjsBffConfig, accessPermissionsRepo) {
        this.bffLoggerService = bffLoggerService;
        this.nestjsBffConfig = nestjsBffConfig;
        this.accessPermissionsRepo = accessPermissionsRepo;
        this.verifyOptions = {
            issuer: nestjsBffConfig.jwt.issuer,
            audience: nestjsBffConfig.http.bffRootUrl,
            algorithms: [nestjsBffConfig.jwt.signingAlgorithm],
        };
    }
    resolve(context) {
        return __awaiter(this, void 0, void 0, function* () {
            return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
                try {
                    this.bffLoggerService.trace('-- JwtMiddleware.resolve', {
                        'req.originalUrl': req ? req.originalUrl : 'null',
                    });
                    yield this.process(req);
                }
                catch (error) {
                    this.bffLoggerService.error('JwtMiddleware Error', error);
                    if (next)
                        next(error);
                }
                if (next)
                    next();
            });
        });
    }
    process(req) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!req || !req.originalUrl) {
                this.bffLoggerService.debug('req.originalUrl is null', core_utils_1.getReqMetadataLite(req));
                return;
            }
            if (this.nestjsBffConfig.http.publicRouteRegex.test(req.originalUrl)) {
                this.bffLoggerService.debug(`Public URL: ${req.originalUrl}`);
                return;
            }
            return this.attachAuthenticatedUserToRequest(req);
        });
    }
    /**
     *
     *
     * @private
     * @param {*} req
     * @returns {Promise<void>}
     * @memberof JwtMiddleware
     */
    attachAuthenticatedUserToRequest(req) {
        return __awaiter(this, void 0, void 0, function* () {
            const jwtToken = this.getJwtBearerTokenFromRequestHeader(req);
            if (!jwtToken)
                return;
            const jwtPayload = jsonwebtoken_1.verify(jwtToken, this.nestjsBffConfig.jwt.jwtPublicKey, this.verifyOptions);
            if (!jwtPayload)
                throw new server_exception_1.BadRequestHttpError('Invalid JWT token', core_utils_1.getReqMetadataLite(req));
            let accessPermissionsEntity;
            try {
                accessPermissionsEntity = yield this.accessPermissionsRepo.findById(jwtPayload.sub, { skipAuthorization: true });
            }
            catch (error) {
                throw new server_exception_1.BadRequestHttpError(`No authentication data found for request: ${req.originalUrl}, jwtPayload: ${JSON.stringify(jwtPayload)}`, error);
            }
            // this.bffLoggerService.debug(`Attaching authorization to request`, {
            //   'req.originalUrl': req.originalUrl,
            //   accessPermissionsEntity,
            //   'org': accessPermissionsEntity.organizations,
            // });
            req.accessPermissions = accessPermissionsEntity;
        });
    }
    /**
     *
     *
     * @private
     * @param {*} req
     * @returns {(string | undefined)}
     * @memberof JwtMiddleware
     */
    getJwtBearerTokenFromRequestHeader(req) {
        const authHdr = req.headers[AttachAuthenticationHttpMiddleware_1.AUTH_HEADER];
        if (!authHdr) {
            // log if authHdr not found
            this.bffLoggerService.debug(`No auth header found for request: ${req.originalUrl}`);
            return undefined;
        }
        const parsedAuthHdr = core_utils_1.parseAuthHeader(authHdr);
        if (!parsedAuthHdr) {
            throw new server_exception_1.BadRequestHttpError(`Malformed auth header found for request: ${req.originalUrl} with authHdr ${authHdr}`, core_utils_1.getReqMetadataLite(req));
        }
        if (parsedAuthHdr.scheme !== AttachAuthenticationHttpMiddleware_1.BEARER_AUTH_SCHEME) {
            throw new server_exception_1.BadRequestHttpError(`Incorrect auth scheme. Bearer expected.  Found ${parsedAuthHdr.scheme}`, core_utils_1.getReqMetadataLite(req));
        }
        if (!parsedAuthHdr.value) {
            throw new server_exception_1.BadRequestHttpError('No auth header value found', core_utils_1.getReqMetadataLite(req));
        }
        return parsedAuthHdr.value;
    }
};
AttachAuthenticationHttpMiddleware.BEARER_AUTH_SCHEME = 'bearer';
AttachAuthenticationHttpMiddleware.AUTH_HEADER = 'authorization';
AttachAuthenticationHttpMiddleware = AttachAuthenticationHttpMiddleware_1 = __decorate([
    common_1.Injectable(),
    __param(1, common_1.Inject(app_shared_constants_1.AppSharedProviderTokens.Config.App)),
    __metadata("design:paramtypes", [logger_shared_service_1.LoggerSharedService, Object, access_permissions_repo_1.AccessPermissionsRepo])
], AttachAuthenticationHttpMiddleware);
exports.AttachAuthenticationHttpMiddleware = AttachAuthenticationHttpMiddleware;
//# sourceMappingURL=attach-authentication.middleware.js.map