"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const authcheck_contract_1 = require("./authcheck.contract");
const org_access_authcheck_1 = require("./org-access.authcheck");
const user_access_authcheck_1 = require("./user-access.authcheck");
class ScopedAccessAuthCheck extends authcheck_contract_1.AuthCheckContract {
    constructor() {
        super(...arguments);
        this.orgAuthCheck = new org_access_authcheck_1.OrgAccessAuthCheck();
        this.userAuthCheck = new user_access_authcheck_1.UserAccessAuthCheck();
    }
    isAuthorized(params) {
        return __awaiter(this, void 0, void 0, function* () {
            // if entity has an orgId, and the accessPermissions don't allow access to this org, then deny authorization (return false)
            if (params.targetResource.orgId) {
                if (!(yield this.orgAuthCheck.isAuthorized(params))) {
                    return false;
                }
            }
            // if entity has a userId, and the accessPermissions don't allow access to this user, then deny authorization (return false)
            if (params.targetResource.userId) {
                if (!(yield this.userAuthCheck.isAuthorized(params))) {
                    return false;
                }
            }
            // if authorization not denied for either org or user access, then allow access (return true)
            return true;
        });
    }
}
exports.ScopedAccessAuthCheck = ScopedAccessAuthCheck;
// Scoped Params Auth Check
//# sourceMappingURL=scoped-access.authcheck.js.map