"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const app_exception_1 = require("../exceptions/app.exception");
const authcheck_contract_1 = require("./authcheck.contract");
const scoped_access_authcheck_1 = require("./scoped-access.authcheck");
const scoped_data_1 = require("./scoped-data");
//
// checks permissions against orgId and userId if these attributes exist on an entity
//
class ScopedEntityAuthCheck extends authcheck_contract_1.AuthCheckContract {
    constructor() {
        super(...arguments);
        this.scopedAccessAuthCheck = new scoped_access_authcheck_1.ScopedAccessAuthCheck();
    }
    isAuthorized(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!params.targetResource || !params.targetResource)
                throw new app_exception_1.AppError('target.resource can not be null');
            const scopedParams = {
                accessPermissions: params.accessPermissions,
                origin: params.origin,
                operation: params.operation,
                targetResource: new scoped_data_1.ScopedData(),
                data: params.data,
            };
            scopedParams.targetResource.userId = params.targetResource['userId'];
            scopedParams.targetResource.orgId = params.targetResource['orgId'];
            return this.scopedAccessAuthCheck.isAuthorized(params);
        });
    }
}
exports.ScopedEntityAuthCheck = ScopedEntityAuthCheck;
//# sourceMappingURL=scoped-entity.authcheck.js.map