"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const testing_utils_1 = require("@nestjs-bff/global-utils-dev/lib/testing.utils");
const test_literals_constants_1 = require("../testing/test-literals.constants");
const scoped_entity_authcheck_1 = require("./scoped-entity.authcheck");
//
// Global Scoped Variables Setup
//
describe('GIVEN a ScopedEntityAuthCheck', () => {
    let scopedEntityAuthCheck;
    beforeAll(() => __awaiter(this, void 0, void 0, function* () {
        scopedEntityAuthCheck = new scoped_entity_authcheck_1.ScopedEntityAuthCheck();
    }));
    //
    // -------------------------------------------
    //
    // FindOne Tests
    //
    // -------------------------------------------
    //
    describe('WHEN calling isAuthorized', () => {
        it(`WITH Org and User Scoped Entity
        WITH no accessPermissions 
        THEN should return false`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: null,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(false);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH valid accessPermissions 
        WITH Org and User Scoped Entity
        THEN should return true`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(true);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH Org and User Scoped Entity
        WITH authorization that doesn't match userId
        WITH authorization that doesn't match orgId
        THEN should return false`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ub1user_ObAdmin,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(false);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH Org and User Scoped Entity
        WITH authorization that doesn't match userId
        WITH OrgRole -> user on related Org
        THEN should return false`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestFooEntityLiterals.FE_Ua1Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(false);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH Org and User Scoped Entity
        WITH authorization that doesn't match userId
        WITH OrgRole -> admin on related Org
        THEN should return true`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua1user_OaAdmin,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(true);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH Org and User Scoped Entity
        WITH authorization that doesn't match userId
        WITH Role -> groupAdmin
        WITH OrgRole -> facilitator on the related organization
        THEN should return true`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Uc1GroupAdmin_OcMember_OaFacilitator,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(true);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH Org and User Scoped Entity
        WITH authorization that doesn't match userId
        WITH Role -> staffAdmin
        THEN should return true`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Uz2StaffAdmin_OzMember,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(true);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH Org and User Scoped Entity
        WITH authorization that doesn't match userId
        WITH Role -> systemAdmin 
        THEN should return true`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Uz1SystemAdmin_OzMember,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestFooEntityLiterals.FE_Ua2Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(true);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH Org Scoped Entity
        WITH authorization that does match the OrgId -> OrgRole user
        THEN should return true`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            const orgScopedEntity = {
                id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                orgId: test_literals_constants_1.TestOrgLiterals.Oa.id,
            };
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember,
                    origin: __filename,
                    targetResource: orgScopedEntity,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(true);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH User Scoped Entity
        WITH authorization that matches the UserId
        THEN should return true`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            const userScopedEntity = {
                id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                userId: test_literals_constants_1.TestUserLiterals.Ua2.id,
            };
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua2User_OaMember,
                    origin: __filename,
                    targetResource: userScopedEntity,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(true);
        }));
        //
        // -------------------------------------------
        //
        it(`WITH User Scoped Entity
        WITH authorization that does not match the UserId
        WITH orgAdmin for owning user
        THEN should return false`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            const userScopedEntity = {
                id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
                userId: test_literals_constants_1.TestUserLiterals.Ua2.id,
            };
            try {
                result = yield scopedEntityAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua1user_OaAdmin,
                    origin: __filename,
                    targetResource: userScopedEntity,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            // note: this should still fail because it is impossible to verify access through org role, if entity not org scoped
            expect(result).toBe(false);
        }));
        //
        // -------------------------------------------
        //
    });
});
//# sourceMappingURL=scoped-entity.authcheck.spec.js.map