"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_literals_constants_1 = require("../testing/test-literals.constants");
const user_access_authcheck_1 = require("./user-access.authcheck");
//
// Global Scoped Variables Setup
//
describe('GIVEN a UserAuthCheck', () => {
    let userAuthCheck;
    beforeAll(() => __awaiter(this, void 0, void 0, function* () {
        userAuthCheck = new user_access_authcheck_1.UserAccessAuthCheck();
    }));
    //
    // -------------------------------------------
    //
    // FindOne Tests
    //
    // -------------------------------------------
    //
    describe('WHEN calling isAuthorized', () => {
        it(`WITH no accessPermissions 
        THEN should return false`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield userAuthCheck.isAuthorized({ accessPermissions: null, origin: __filename, targetResource: test_literals_constants_1.TestScopedDataLiterals.Sc_Ua1Oa });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(false);
        }));
    });
    //
    // -------------------------------------------
    //
    describe('WHEN calling isAuthorized', () => {
        it(`WITH valid accessPermissions 
        THEN should return true`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield userAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua1user_OaAdmin,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestScopedDataLiterals.Sc_Ua1Oa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).toBeUndefined();
            expect(result).toBe(true);
        }));
    });
    //
    // -------------------------------------------
    //
    describe('WHEN calling isAuthorized', () => {
        it(`WITH null targetUserId 
        THEN should throw error`, () => __awaiter(this, void 0, void 0, function* () {
            let error;
            let result;
            try {
                result = yield userAuthCheck.isAuthorized({
                    accessPermissions: test_literals_constants_1.TestAuthorizationLiterals.Az_Ua1user_OaAdmin,
                    origin: __filename,
                    targetResource: test_literals_constants_1.TestScopedDataLiterals.Sc_UxOa,
                });
            }
            catch (e) {
                error = e;
            }
            expect(error).not.toBeUndefined();
            expect(result).not.toBe(true);
        }));
    });
});
//# sourceMappingURL=user-access.authcheck.spec.js.map