"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const class_validator_1 = require("class-validator");
const stringify = require("json-stable-stringify");
const hash_utils_1 = require("../utils/hash.utils");
/**
 * @description: Utility functions to support caching operations
 * @notes:
 *  - Uses json-stable-stringify (instead of JSON.Stringify) for determanistic string generation - regardless of parameter ordering
 *  - Uses custom hash function as significantly faster than cryptogaphic hashes
 */
class CachingUtils {
    static makeCacheKeyFromId(entityId) {
        this.validator.isMongoId(entityId);
        return this.makeCacheKeyFromProperty(entityId, 'id');
    }
    static makeCacheKeyFromProperty(propertyName, propertyValue) {
        this.validator.isNotEmpty(propertyValue);
        this.validator.isNotEmpty(propertyName);
        return `CacheKey-${propertyName}-${propertyValue}`;
    }
    static makeCacheKeyFromObject(object) {
        return hash_utils_1.hash(stringify(object)).toString();
    }
}
CachingUtils.validator = new class_validator_1.Validator();
exports.CachingUtils = CachingUtils;
//# sourceMappingURL=caching.utils.js.map