"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const testing_utils_1 = require("@nestjs-bff/global-utils-dev/lib/testing.utils");
const foo_entity_1 = require("../../domain/_foo/model/foo.entity");
const logging_shared_module_1 = require("../logging/logging.shared.module");
const caching_utils_1 = require("./caching.utils");
const performanceTestIterations = 1000;
// @ts-ignore
const logger = logging_shared_module_1.getLogger();
describe('CachingUtils', () => {
    describe('makeCacheKeyFromId', () => {
        it('should return a string', () => __awaiter(this, void 0, void 0, function* () {
            const result = caching_utils_1.CachingUtils.makeCacheKeyFromId('507f191e810c19729de860ea');
            // logger.debug('makeCacheKeyFromId-result', result);
            expect(typeof result).toBe('string');
            expect(result.length).toBeGreaterThan(0);
        }));
    });
    describe('makeCacheKeyFromProperty', () => {
        it('should return a string', () => __awaiter(this, void 0, void 0, function* () {
            const result = caching_utils_1.CachingUtils.makeCacheKeyFromProperty('foo', 'bar');
            // logger.debug('makeCacheKeyFromProperty-result', result);
            expect(typeof result).toBe('string');
            expect(result.length).toBeGreaterThan(0);
        }));
    });
    describe('makeCacheKeyFromObject', () => {
        describe('with basic object', () => {
            it('should return a string', () => __awaiter(this, void 0, void 0, function* () {
                const result = caching_utils_1.CachingUtils.makeCacheKeyFromObject({
                    foo: 'foo',
                    bar: { baz: 'baz' },
                });
                // logger.debug('makeCacheKeyFromObject-result', result);
                expect(typeof result).toBe('string');
                expect(result.length).toBeGreaterThan(0);
            }));
            it('should always return the same out from the same input - regardless of param ordering', () => __awaiter(this, void 0, void 0, function* () {
                const result1 = caching_utils_1.CachingUtils.makeCacheKeyFromObject({
                    foo: 'foo',
                    bar: { baz: 'baz' },
                });
                const result2 = caching_utils_1.CachingUtils.makeCacheKeyFromObject({
                    bar: { baz: 'baz' },
                    foo: 'foo',
                });
                // logger.debug('makeCacheKeyFromObject-result', { result1, result2 });
                expect(result1).toEqual(result2);
            }));
            it('should return different results for different input', () => __awaiter(this, void 0, void 0, function* () {
                const result1 = caching_utils_1.CachingUtils.makeCacheKeyFromObject({
                    foo: 'foo',
                    bar: { baz: 'baz' },
                });
                const result2 = caching_utils_1.CachingUtils.makeCacheKeyFromObject({
                    foo: 'foo',
                    bar: { baz: 'bag' },
                });
                // logger.debug('makeCacheKeyFromObject-result', { result1, result2 });
                expect(result1).not.toEqual(result2);
            }));
        });
    });
    describe('with full query condition', () => {
        it('should return a string', () => __awaiter(this, void 0, void 0, function* () {
            const qc = new foo_entity_1.FooEntity();
            qc.id = '507f191e810c19729de860ea';
            qc.orgId = '607f191e810c19729de860ea';
            qc.userId = '707f191e810c19729de860ea';
            qc.alwaysDefinedSlug = 'foo';
            const result = caching_utils_1.CachingUtils.makeCacheKeyFromObject(qc);
            // logger.debug('makeCacheKeyFromObject-result', result);
            expect(typeof result).toBe('string');
            expect(result.length).toBeGreaterThan(0);
        }));
    });
    describe('with partial query condition', () => {
        it('should return a string', () => __awaiter(this, void 0, void 0, function* () {
            const qc = new foo_entity_1.FooEntity();
            qc.orgId = '607f191e810c19729de860ea';
            qc.alwaysDefinedSlug = 'foo';
            const result = caching_utils_1.CachingUtils.makeCacheKeyFromObject(qc);
            // logger.debug('makeCacheKeyFromObject-result', result);
            expect(typeof result).toBe('string');
            expect(result.length).toBeGreaterThan(0);
        }));
    });
    describe('makeCacheKeyFromObject - performance', () => {
        it('should be fast', () => __awaiter(this, void 0, void 0, function* () {
            const makeCachKeyFunc = () => {
                caching_utils_1.CachingUtils.makeCacheKeyFromObject({
                    foo: 'foo',
                    bar: { baz: 'baz' },
                });
            };
            const processingTime = testing_utils_1.TestingUtils.measureExecutionTime(makeCachKeyFunc, performanceTestIterations);
            // logger.debug(`makeCachKeyFunc ${performanceTestIterations} iterations`, processingTime);
            expect(processingTime).toBeLessThan(1000);
        }));
    });
});
//# sourceMappingURL=caching.utils.spec.js.map