"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var LoggerWinstonSharedService_1;
"use strict";
const common_1 = require("@nestjs/common");
const FileSystem = require("fs");
const winston = require("winston");
const app_shared_constants_1 = require("../app/app.shared.constants");
let LoggerWinstonSharedService = LoggerWinstonSharedService_1 = class LoggerWinstonSharedService {
    constructor(nestjsBffConfig) {
        this.nestjsBffConfig = nestjsBffConfig;
        this.transports = [];
        this.appName = nestjsBffConfig.appName;
        const logLevel = this.nestjsBffConfig.logging.winston.level;
        const logTransport = this.nestjsBffConfig.logging.winston.transports;
        for (const key in logTransport) {
            if (logTransport.hasOwnProperty(key)) {
                if (key === LoggerWinstonSharedService_1.TRANSPORT_KEY_CONSOLE) {
                    this.configureConsoleTransport(logTransport[key], logLevel);
                }
                else if (key === LoggerWinstonSharedService_1.TRANSPORT_KEY_FILE) {
                    this.configureFileTransport(logTransport[key]);
                }
            }
        }
        this.logger = winston.createLogger({
            transports: this.transports,
        });
    }
    log(msg, ...logObjects) {
        this.info(msg, ...logObjects);
    }
    info(msg, ...logObjects) {
        this.logger.info(msg, ...logObjects);
    }
    error(msg, ...logObjects) {
        this.logger.error(msg, ...logObjects);
    }
    warn(msg, ...logObjects) {
        this.logger.warn(msg, ...logObjects);
    }
    debug(msg, ...logObjects) {
        this.logger.debug(msg, ...logObjects);
    }
    trace(msg, ...logObjects) {
        this.logger.verbose(msg, ...logObjects);
    }
    /**
     * Configuring console transport
     */
    configureConsoleTransport(transport, logLevel) {
        const options = Object.assign({
            level: logLevel,
        }, transport);
        this.transports.push(new winston.transports.Console(options));
    }
    /**
     * Configuring file transport
     */
    configureFileTransport(transport) {
        const targetFileList = this.nestjsBffConfig.logging.winston.target;
        // Creating log directory if it does not exist
        if (!FileSystem.existsSync(this.nestjsBffConfig.logging.logDir)) {
            FileSystem.mkdirSync(this.nestjsBffConfig.logging.logDir);
        }
        for (const key in targetFileList) {
            if (targetFileList.hasOwnProperty(key)) {
                const options = Object.assign({
                    name: key,
                    level: key,
                    filename: `${this.nestjsBffConfig.logging.logDir}/${this.appName + '-' + targetFileList[key]}`,
                }, transport);
                this.transports.push(new winston.transports.File(options));
            }
        }
    }
};
LoggerWinstonSharedService.TRANSPORT_KEY_CONSOLE = 'console';
LoggerWinstonSharedService.TRANSPORT_KEY_FILE = 'file';
LoggerWinstonSharedService = LoggerWinstonSharedService_1 = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(app_shared_constants_1.AppSharedProviderTokens.Config.App)),
    __metadata("design:paramtypes", [Object])
], LoggerWinstonSharedService);
exports.LoggerWinstonSharedService = LoggerWinstonSharedService;
//# sourceMappingURL=logger-winston.shared.service.js.map