import { Model } from 'mongoose';
import { LoggerSharedService } from '../../shared/logging/logger.shared.service';
import { IMigration } from './interfaces/migration.interface';
export declare class MigrationsSharedService {
    private readonly bffLoggerService;
    private readonly migrationModel;
    private readonly relativePaths;
    private readonly absolutePaths;
    private readonly template;
    constructor(bffLoggerService: LoggerSharedService, migrationModel: Model<IMigration>);
    /**
     *
     * @param migrationGroup
     */
    autoRunMigrations(migrationGroup: string): Promise<void>;
    /**
     *
     * @param direction {string}
     * @param migrationRelativePath {string}
     */
    runCustomMigration(direction: string | undefined, migrationRelativePath: string): Promise<void>;
    getCustomMigrationFilePath(migrationFilename: string): string;
    /**
     * Create a new migration
     * @param {string} migrationName
     * @returns {Promise<Object>} A promise of the Migration created
     */
    create(migrationGroup: string, migrationName: string): Promise<void>;
    /**
     * Runs migrations up to or down to a given migration name.
     * If no name given, all migrations are run.
     *
     * @param migrationName
     * @param direction
     */
    runMigration(migrationGroup: string, direction?: string, migrationName?: string): Promise<void>;
    /**
     * Looks at the file system migrations and import any migrations that are
     * on the file system but missing in the database into the database
     *
     * This functionality is opposite of prune()
     */
    sync(migrationGroup: string, interactive: boolean): Promise<void>;
    /**
     * Opposite of sync().
     * Removes files in the database which don't exist in the migrations directory.
     */
    prune(migrationGroup: string, interactive: boolean): Promise<any>;
    /**
     * Lists the current migrations and their statuses
     * @returns {Promise<Array<Object>>}
     * @example
     *   [
     *    { name: 'my-migration', filename: '149213223424_my-migration.js', state: 'up' },
     *    { name: 'add-cows', filename: '149213223453_add-cows.js', state: 'down' }
     *   ]
     */
    list(migrationGroup: string): Promise<void>;
    /**
     *
     */
    private compareFileSystemWithDB;
    /**
     *
     * @param migrationToImport
     */
    private getFilenameParts;
    /**
     *
     * @param migrationFilePath
     * @param direction
     * @param migrationFilename
     */
    private getDirectionalMigrationFunction;
    private runMigrationFunction;
}
//# sourceMappingURL=migrations.shared.service.d.ts.map