"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const fs = require("fs");
const inquirer = require("inquirer");
const mongoose_1 = require("mongoose");
const path = require("path");
const tsnode = require("ts-node");
const logger_shared_service_1 = require("../../shared/logging/logger.shared.service");
const migration_interface_1 = require("./interfaces/migration.interface");
const migrations_shared_constants_1 = require("./migrations.shared.constants");
// mongooseSet('debug', true);
let MigrationsSharedService = class MigrationsSharedService {
    constructor(bffLoggerService, migrationModel) {
        this.bffLoggerService = bffLoggerService;
        this.migrationModel = migrationModel;
        this.relativePaths = {
            migrationScriptsRoot: 'src/migrations',
            customScripts: 'src/migrations/custom',
            migrationsTemplate: '../../../templates/migrationTemplate.ts',
        };
        this.absolutePaths = {
            migrationScriptsRoot: '',
            customScripts: '',
            migrationsTemplate: '',
        };
        // set absolute paths
        this.absolutePaths.migrationScriptsRoot = path.resolve(process.cwd(), this.relativePaths.migrationScriptsRoot);
        this.absolutePaths.customScripts = path.resolve(process.cwd(), this.relativePaths.customScripts);
        this.absolutePaths.migrationsTemplate = path.resolve(__dirname, this.relativePaths.migrationsTemplate);
        // initialize objects
        this.template = fs.readFileSync(this.absolutePaths.migrationsTemplate, 'utf-8');
    }
    /**
     *
     * @param migrationGroup
     */
    autoRunMigrations(migrationGroup) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.bffLoggerService.info(`About to run migrations for group: ${migrationGroup}`);
                yield this.sync(migrationGroup, false);
                yield this.runMigration(migrationGroup);
                this.bffLoggerService.info(`Ran migrations for group: ${migrationGroup}`);
            }
            catch (error) {
                this.bffLoggerService.error(error);
            }
        });
    }
    /**
     *
     * @param direction {string}
     * @param migrationRelativePath {string}
     */
    runCustomMigration(direction = 'up', migrationRelativePath) {
        return __awaiter(this, void 0, void 0, function* () {
            tsnode.register({ transpileOnly: true });
            const migrationFilePath = this.getCustomMigrationFilePath(migrationRelativePath);
            try {
                const directionalMigrationFunction = yield this.getDirectionalMigrationFunction(migrationFilePath, direction, migrationRelativePath);
                if (!directionalMigrationFunction)
                    return;
                yield this.runMigrationFunction(directionalMigrationFunction);
                this.bffLoggerService.log(`Migration successfully ran: ${migrationRelativePath} (${direction.toUpperCase()})`);
            }
            catch (e) {
                this.bffLoggerService.error(`Failed to run migration ${migrationFilePath} due to an error.`, e);
                this.bffLoggerService.warn(`Not continuing. Make sure your data is in consistent state`);
                throw e;
            }
        });
    }
    getCustomMigrationFilePath(migrationFilename) {
        return path.join(this.absolutePaths.customScripts, migrationFilename);
    }
    /**
     * Create a new migration
     * @param {string} migrationName
     * @returns {Promise<Object>} A promise of the Migration created
     */
    create(migrationGroup, migrationName) {
        return __awaiter(this, void 0, void 0, function* () {
            // validate migration doesn't already exist
            const existingMigration = yield this.migrationModel.findOne({
                name: migrationName,
            });
            if (!!existingMigration) {
                throw new Error(`There is already a migration with name '${migrationName}' in the database`);
            }
            const migrationCreatedDate = Date.now();
            // create file system file
            const newMigrationFileName = `${migrationCreatedDate}-${migrationName}.ts`;
            const newMigrationFilePath = path.join(this.absolutePaths.migrationScriptsRoot, migrationGroup, newMigrationFileName);
            fs.writeFileSync(newMigrationFilePath, this.template);
            // create instance in db
            const createdMigration = new this.migrationModel({
                name: migrationName,
                createdAt: migrationCreatedDate,
            });
            yield createdMigration.save();
            this.bffLoggerService.log(`Created migration ${migrationName} in ${newMigrationFilePath}.`);
        });
    }
    /**
     * Runs migrations up to or down to a given migration name.
     * If no name given, all migrations are run.
     *
     * @param migrationName
     * @param direction
     */
    runMigration(migrationGroup, direction = 'up', migrationName) {
        return __awaiter(this, void 0, void 0, function* () {
            const untilMigration = migrationName
                ? yield this.migrationModel.findOne({ name: migrationName })
                : yield this.migrationModel.findOne().sort({ createdAt: -1 });
            if (!untilMigration) {
                if (migrationName) {
                    throw new ReferenceError('Could not find that migration in the database');
                }
                // no migrations to run
                this.bffLoggerService.info(`There are no migrations to run`);
                return;
            }
            let query;
            if (direction === 'down') {
                query = {
                    createdAt: { $gte: untilMigration.createdAt },
                    state: 'up',
                };
            }
            else {
                query = {
                    createdAt: { $lte: untilMigration.createdAt },
                    state: 'down',
                };
            }
            const sortDirection = direction === 'up' ? 1 : -1;
            const migrationsToRun = yield this.migrationModel.find(query).sort({ createdAt: sortDirection });
            if (!migrationsToRun.length) {
                this.bffLoggerService.warn('There are no migrations to run');
                this.bffLoggerService.warn(`Current Migrations' Statuses: `);
                yield this.list(migrationGroup);
            }
            let numMigrationsRun = 0;
            tsnode.register({ transpileOnly: true });
            for (const migration of migrationsToRun) {
                const migrationFilePath = path.join(this.absolutePaths.migrationScriptsRoot, migrationGroup, migration.filename);
                try {
                    const directionalMigrationFunction = yield this.getDirectionalMigrationFunction(migrationFilePath, direction, migration.filename);
                    if (!directionalMigrationFunction)
                        return;
                    // run migration
                    yield this.runMigrationFunction(directionalMigrationFunction);
                    // update migrations table
                    this.bffLoggerService.debug(`Update migration table for migration: ${migration.name}`);
                    const completed = yield this.migrationModel.updateOne({ name: migration.name }, { state: direction });
                    this.bffLoggerService.log(`migration models updated: ${JSON.stringify(completed)}`);
                    // log completed
                    this.bffLoggerService.log(`${migration.filename} (${direction.toUpperCase()})`);
                    numMigrationsRun++;
                }
                catch (err) {
                    this.bffLoggerService.error(`Failed to run migration ${migrationFilePath} due to an error.`, err.stack);
                    this.bffLoggerService.warn(`Not continuing. Make sure your data is in consistent state`);
                    throw err;
                }
            }
            this.bffLoggerService.log(`Migrations run: ${numMigrationsRun}`);
        });
    }
    /**
     * Looks at the file system migrations and import any migrations that are
     * on the file system but missing in the database into the database
     *
     * This functionality is opposite of prune()
     */
    sync(migrationGroup, interactive) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { filesNotInDb } = yield this.compareFileSystemWithDB(migrationGroup);
                let migrationsToImport = filesNotInDb;
                this.bffLoggerService.log('Synchronizing file system migrations with database...');
                if (migrationsToImport.length === 0) {
                    this.bffLoggerService.log('There are no migrations to synchronize');
                }
                else if (interactive) {
                    yield inquirer
                        .prompt({
                        type: 'checkbox',
                        message: 'The following migrations exist in the migrations folder but not in the database. Select the ones you want to import into the database',
                        name: 'migrationsToImport',
                        choices: filesNotInDb,
                    })
                        .then(answers => {
                        migrationsToImport = answers.migrationsToImport;
                    });
                }
                const promises = new Array();
                for (const migrationToImport of migrationsToImport) {
                    promises.push(new Promise((resolve, reject) => {
                        const { filePath, migrationName, timestamp } = this.getFilenameParts(migrationGroup, migrationToImport);
                        this.bffLoggerService.log(`Adding migration ${filePath} into database require(file system. State is ` + `DOWN`);
                        this.migrationModel
                            .create({
                            name: migrationName,
                            createdAt: timestamp,
                        })
                            .then(createdMigration => {
                            this.bffLoggerService.log(`Migration created: ${JSON.stringify(createdMigration.toJSON())}`);
                            resolve(createdMigration);
                        });
                    }));
                    yield Promise.all(promises).catch(reason => this.bffLoggerService.error(`Could not synchronize migrations in the migrations folder up to the database`, reason));
                }
            }
            catch (error) {
                this.bffLoggerService.error(`Could not synchronize migrations in the migrations folder up to the database.`, error.stack);
                throw error;
            }
        });
    }
    /**
     * Opposite of sync().
     * Removes files in the database which don't exist in the migrations directory.
     */
    prune(migrationGroup, interactive) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { dbMigrationsNotOnFs } = yield this.compareFileSystemWithDB(migrationGroup);
                let migrationsToDelete = dbMigrationsNotOnFs.map(m => m.name);
                if (migrationsToDelete.length === 0) {
                    this.bffLoggerService.log('There are no migrations to prune');
                }
                else if (interactive) {
                    yield inquirer
                        .prompt({
                        type: 'checkbox',
                        message: 'The following migrations exist in the database but not in the migrations folder. Select the migrations you want to remove from the database.',
                        name: 'migrationsToDelete',
                        choices: migrationsToDelete,
                    })
                        .then(answers => {
                        migrationsToDelete = answers.migrationsToDelete;
                    });
                }
                const migrationsToDeleteDocs = yield this.migrationModel
                    .find({
                    name: { $in: migrationsToDelete },
                })
                    .lean();
                if (migrationsToDelete.length) {
                    this.bffLoggerService.log(`Removing migration(s) ${migrationsToDelete.join(', ')} from database`);
                    yield this.migrationModel.remove({
                        name: { $in: migrationsToDelete },
                    });
                }
                return migrationsToDeleteDocs;
            }
            catch (error) {
                this.bffLoggerService.error(`Could not prune extraneous migrations from database.`);
                throw error;
            }
        });
    }
    /**
     * Lists the current migrations and their statuses
     * @returns {Promise<Array<Object>>}
     * @example
     *   [
     *    { name: 'my-migration', filename: '149213223424_my-migration.js', state: 'up' },
     *    { name: 'add-cows', filename: '149213223453_add-cows.js', state: 'down' }
     *   ]
     */
    list(migrationGroup) {
        return __awaiter(this, void 0, void 0, function* () {
            const { migrationsInDatabase, migrationsInFolder, filesNotInDb, dbMigrationsNotOnFs } = yield this.compareFileSystemWithDB(migrationGroup);
            this.bffLoggerService.log(`There are ${migrationsInDatabase.length} migration documents in the database, and ${migrationsInFolder.length} migration files on the file system.`);
            if (migrationsInDatabase.length > 0) {
                this.bffLoggerService.log('\n');
                this.bffLoggerService.log(`The following migrations were detected:`);
                migrationsInDatabase.map(m => {
                    this.bffLoggerService.log(`${m.filename} (${m.state === migration_interface_1.MigrationState.Up ? 'UP:' : 'DOWN'})`);
                });
            }
            if (dbMigrationsNotOnFs.length > 0) {
                this.bffLoggerService.log('\n');
                this.bffLoggerService.log(`The following migrations exist in the database, but not the filesystem. (Run the migrations-prune command to remove these from the database): `);
                dbMigrationsNotOnFs.map(m => {
                    this.bffLoggerService.log(`${m.filename} (${m.state === migration_interface_1.MigrationState.Up ? 'UP' : 'DOWN'})`);
                });
            }
            if (filesNotInDb.length > 0) {
                this.bffLoggerService.log('\n');
                this.bffLoggerService.log(`The following migrations exist on the filesystem, but not the database. (Run the migrations-sync command to add these to the database): `);
                filesNotInDb.map(m => {
                    this.bffLoggerService.log(m);
                });
            }
        });
    }
    /**
     *
     */
    compareFileSystemWithDB(migrationGroup) {
        return __awaiter(this, void 0, void 0, function* () {
            const filesInMigrationFolder = fs.readdirSync(path.join(this.absolutePaths.migrationScriptsRoot, migrationGroup));
            const migrationsInDatabase = yield this.migrationModel.find({}).sort({ createdAt: 1 });
            // this.bffLoggerService.debug('compareFileSystemWithDB.migrationsInDatabase', migrationsInDatabase);
            const migrationsInFolder = filesInMigrationFolder
                .filter(file => /\d{13,}\-.+.(ts)$/.test(file))
                .map(filename => {
                const fileCreatedAt = +filename.split('-')[0];
                const existsInDatabase = migrationsInDatabase.some(m => filename === m.filename);
                return { createdAt: fileCreatedAt, filename, existsInDatabase };
            });
            // this.bffLoggerService.debug('compareFileSystemWithDB.migrationsInFolder', filesInMigrationFolder, migrationsInFolder);
            const filesNotInDb = migrationsInFolder.filter(migrationInFolder => migrationInFolder.existsInDatabase === false).map(f => f.filename);
            const dbMigrationsNotOnFs = migrationsInDatabase.filter(migrationInDB => {
                return !migrationsInFolder.find(migrationInFolder => migrationInFolder.filename === migrationInDB.filename);
            });
            return {
                migrationsInDatabase,
                migrationsInFolder,
                filesNotInDb,
                dbMigrationsNotOnFs,
            };
        });
    }
    /**
     *
     * @param migrationToImport
     */
    getFilenameParts(migrationGroup, migrationToImport) {
        const filePath = path.join(this.absolutePaths.migrationScriptsRoot, migrationGroup, migrationToImport);
        const timestampSeparatorIndex = migrationToImport.indexOf('-');
        const timestamp = migrationToImport.slice(0, timestampSeparatorIndex);
        const migrationName = migrationToImport.slice(timestampSeparatorIndex + 1, migrationToImport.lastIndexOf('.'));
        return { filePath, migrationName, timestamp };
    }
    /**
     *
     * @param migrationFilePath
     * @param direction
     * @param migrationFilename
     */
    getDirectionalMigrationFunction(migrationFilePath, direction, migrationFilename) {
        return __awaiter(this, void 0, void 0, function* () {
            let migrationFunctions;
            // tslint:disable-next-line:whitespace
            migrationFunctions = yield Promise.resolve().then(() => require(migrationFilePath));
            this.bffLoggerService.debug(migrationFunctions);
            if (!migrationFunctions) {
                this.bffLoggerService.error(`The migration file '${migrationFilePath}' could not be loaded.`);
                return null;
            }
            const directionalMigrationFunction = migrationFunctions[direction];
            if (!directionalMigrationFunction) {
                this.bffLoggerService.error(`The ${direction} export is not defined in ${migrationFilename}.`);
                return null;
            }
            return directionalMigrationFunction;
        });
    }
    runMigrationFunction(directionalMigrationFunction) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve, reject) => {
                const callPromise = directionalMigrationFunction.call(null, this.migrationModel.collection.conn, this.bffLoggerService, function callback(err) {
                    if (err)
                        return reject(err);
                    resolve();
                });
                if (callPromise && typeof callPromise.then === 'function') {
                    callPromise.then(resolve).catch(reject);
                }
            });
        });
    }
};
MigrationsSharedService = __decorate([
    common_1.Injectable(),
    __param(1, common_1.Inject(migrations_shared_constants_1.MigrationsProviderTokens.Models.Migration)),
    __metadata("design:paramtypes", [logger_shared_service_1.LoggerSharedService,
        mongoose_1.Model])
], MigrationsSharedService);
exports.MigrationsSharedService = MigrationsSharedService;
//# sourceMappingURL=migrations.shared.service.js.map