"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const roles_constants_1 = require("@nestjs-bff/global-contracts/lib/shared/authorization/roles.constants");
const testing_utils_1 = require("@nestjs-bff/global-utils-dev/lib/testing.utils");
exports.TestOrgLiterals = {
    Oa: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        name: 'org A',
        slug: 'org-a',
    },
    Ob: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        name: 'org B',
        slug: 'ob',
    },
    Oc: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        name: 'org C',
        slug: 'oc',
    },
    Oz: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        name: 'org Z',
        slug: 'oz',
    },
};
exports.TestUserLiterals = {
    Ua1: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        username: 'user A1',
        displayName: 'ua1',
    },
    Ua2: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        username: 'user A2',
        displayName: 'ua2',
    },
    Ub1: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        username: 'user B1',
        displayName: 'ub1',
    },
    Ub2: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        username: 'user B2',
        displayName: 'ub2',
    },
    Uc1: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        username: 'user C1',
        displayName: 'uc1',
    },
    Uz1: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        username: 'user Z1',
        displayName: 'uz1',
    },
    Uz2: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        username: 'user Z2',
        displayName: 'uz2',
    },
};
exports.TestFooEntityLiterals = {
    FE_Ua1Oa: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        name: 'Foo Entity Ua1Oa',
        alwaysDefinedSlug: 'fe-ua1oa',
        orgId: exports.TestOrgLiterals.Oa.id,
        userId: exports.TestUserLiterals.Ua1.id,
    },
    FE_Ua2Oa: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        name: 'Foo Entity Ua2Oa',
        alwaysDefinedSlug: 'fe-ua2oa',
        orgId: exports.TestOrgLiterals.Oa.id,
        userId: exports.TestUserLiterals.Ua2.id,
    },
    FE_Ub1Ob: {
        id: testing_utils_1.TestingUtils.generateMongoObjectIdString(),
        name: 'Foo Entity Ub1Ob',
        alwaysDefinedSlug: 'fe-ub1ob',
        orgId: exports.TestOrgLiterals.Ob.id,
        userId: exports.TestUserLiterals.Ub1.id,
    },
};
exports.TestAuthorizationLiterals = {
    Az_Ua1user_OaAdmin: {
        userId: exports.TestUserLiterals.Ua1.id,
        roles: [roles_constants_1.Roles.user],
        organizations: [{ primary: true, orgId: exports.TestOrgLiterals.Oa.id, organizationRoles: [roles_constants_1.OrganizationRoles.admin] }],
    },
    Az_Ua2User_OaMember: {
        userId: exports.TestUserLiterals.Ua2.id,
        roles: [roles_constants_1.Roles.user],
        organizations: [{ primary: true, orgId: exports.TestOrgLiterals.Oa.id, organizationRoles: [roles_constants_1.OrganizationRoles.member] }],
    },
    Az_Ub1user_ObAdmin: {
        userId: exports.TestUserLiterals.Ub1.id,
        roles: [roles_constants_1.Roles.user],
        organizations: [{ primary: true, orgId: exports.TestOrgLiterals.Ob.id, organizationRoles: [roles_constants_1.OrganizationRoles.member] }],
    },
    Az_Ub2User_ObMember: {
        userId: exports.TestUserLiterals.Ub2.id,
        roles: [roles_constants_1.Roles.user],
        organizations: [{ primary: true, orgId: exports.TestOrgLiterals.Oa.id, organizationRoles: [roles_constants_1.OrganizationRoles.admin] }],
    },
    Az_Uc1GroupAdmin_OcMember_OaFacilitator: {
        userId: exports.TestUserLiterals.Uc1.id,
        roles: [roles_constants_1.Roles.groupAdmin],
        organizations: [
            { primary: true, orgId: exports.TestOrgLiterals.Oc.id, organizationRoles: [roles_constants_1.OrganizationRoles.member] },
            { primary: false, orgId: exports.TestOrgLiterals.Oa.id, organizationRoles: [roles_constants_1.OrganizationRoles.facilitator] },
        ],
    },
    Az_Uz1SystemAdmin_OzMember: {
        userId: exports.TestUserLiterals.Uz1.id,
        roles: [roles_constants_1.Roles.systemAdmin],
        organizations: [{ primary: true, orgId: exports.TestOrgLiterals.Oa.id, organizationRoles: [roles_constants_1.OrganizationRoles.member] }],
    },
    Az_Uz2StaffAdmin_OzMember: {
        userId: exports.TestUserLiterals.Uz2.id,
        roles: [roles_constants_1.Roles.staffAdmin],
        organizations: [{ primary: true, orgId: exports.TestOrgLiterals.Oa.id, organizationRoles: [roles_constants_1.OrganizationRoles.member] }],
    },
};
exports.TestScopedDataLiterals = {
    Sc_Ua1Oa: {
        userId: exports.TestUserLiterals.Ua1.id,
        orgId: exports.TestOrgLiterals.Oa.id,
    },
    Sc_Ua2Oa: {
        userId: exports.TestUserLiterals.Ua2.id,
        orgId: exports.TestOrgLiterals.Oa.id,
    },
    Sc_Ua2Ox: {
        userId: exports.TestUserLiterals.Ua2.id,
    },
    Sc_UxOa: {
        orgId: exports.TestOrgLiterals.Oa.id,
    },
    Sc_UxOx: {},
};
//# sourceMappingURL=test-literals.constants.js.map